/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.storage;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.Collections;
import java.util.Set;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.CropDefinitionManager;
import nl.cloudfarming.client.crop.model.CropManager;
import org.openide.util.Lookup;

/**
 * Storage object to store cropdefinitions
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class CropDefinitionStorage implements Serializable {

    private static final long serialVersionUID = 1L;
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    private static CropDefinitionStorage instance;
    public static final String PROP_CROP_DEFINITION_ADDREMOVE = "cropDefinitionAddRemove";

    /**
     * Get a singleton cropstorage instance
     *
     * @return Singleton instance
     */
    public static CropDefinitionStorage getInstance() {
        if (instance == null) {
            instance = new CropDefinitionStorage();
        }
        return instance;
    }

    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    /**
     * Get an unmodifiable list of cropdefinitions in the cropstorage
     *
     * @return The list of cropdefinitions
     */
    public Set<CropDefinition> getCropDefinitions() {
        return Collections.unmodifiableSet(getCropDefinitionManager().findAll());
    }

    /**
     * Remove a cropDefinition from the cropstorage
     *
     * @param cropDefinition Cropdefinition to remove
     */
    public void removeCropDefinition(CropDefinition cropDefinition) {
        getCropDefinitionManager().remove(cropDefinition); 
        getPropertyChangeSupport().firePropertyChange(PROP_CROP_DEFINITION_ADDREMOVE, cropDefinition, null);
    }

    /**
     * Add the specified cropDefinition to the cropStorage and fire
     * propertychange
     *
     * @param cropDefinition
     */
    public void addCropDefinition(CropDefinition cropDefinition) {
        getCropManager().save(cropDefinition.getCrop());
        
        getCropDefinitionManager().save(cropDefinition);
        getPropertyChangeSupport().firePropertyChange(PROP_CROP_DEFINITION_ADDREMOVE, null, cropDefinition);
    }
    
    private CropManager getCropManager() {
        return Lookup.getDefault().lookup(CropManager.class);
    }

    /**
     * Add a propertychangelistener to this instance
     *
     * @param property To listen to
     * @param listener The listener to register
     */
    public void addPropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().addPropertyChangeListener(property, listener);
    }

    /**
     * Add a propertychangelistener to this instance, listen to everything
     *
     * @param listener The listener to register
     */
    public void addPropertyChangeListener(PropertyChangeListener pcl) {
        getPropertyChangeSupport().addPropertyChangeListener(pcl);
    }

    /**
     * Remove a propertychangelistener from this instance
     *
     * @param property The property the registered listener is listening to
     * @param listener The registered listener to remove
     */
    public void removePropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }

    private CropDefinitionManager getCropDefinitionManager() {
        return Lookup.getDefault().lookup(CropDefinitionManager.class);
    }
    
    public void saveAll() {
        Set<CropDefinition> cropDefinitions = getCropDefinitionManager().findAll();
        for(CropDefinition cropDef : cropDefinitions) {
            getCropManager().save(cropDef.getCrop());
            getCropDefinitionManager().save(cropDef);
        }
    }
}
