/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.storage;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import nl.cloudfarming.client.crop.model.CropBean;

/**
 * Add an instance of this class as PropertyChangeListener to CropBeans to track
 * which CropBeans have changed properties and be able to undo property changes.
 *
 * @author Wytse Visser
 */
public class PropertyChangeTracker implements PropertyChangeListener {

    private final Map<CropBean, OriginalProperties> properties = new ConcurrentHashMap<>();

    public final void reset() {
        properties.clear();
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        OriginalProperties originalProperties = properties.get((CropBean) evt.getSource());
        if(originalProperties == null){
            originalProperties = new OriginalProperties();
            properties.put((CropBean) evt.getSource(), originalProperties);
        }
        originalProperties.propertyChange(evt);
    }

    public Set<CropBean> getUpdatedCropBeans() {
        for (Iterator<Map.Entry<CropBean,OriginalProperties>> it = properties.entrySet().iterator(); it.hasNext();) {
            Map.Entry<CropBean,OriginalProperties> entry = it.next();
            if (entry.getValue().isEmpty()) {
                it.remove();
            }
        }
        return properties.keySet();
    }

    public void undoPropertyChanges() {
        for (CropBean cropBean : getUpdatedCropBeans()) {
            undoNameChange(cropBean);
            undoSourceChange(cropBean);
            undoColorChange(cropBean);
        }
        reset();
    }

    private void undoNameChange(CropBean cropBean) {
        if (properties.get(cropBean).containsKey(CropBean.PROP_NAME)) {
            cropBean.setName((String) properties.get(cropBean).get(CropBean.PROP_NAME));
        }
    }

    private void undoSourceChange(CropBean cropBean) {
        if (properties.get(cropBean).containsKey(CropBean.PROP_SOURCE)) {
            cropBean.setSource((String) properties.get(cropBean).get(CropBean.PROP_SOURCE));
        }
    }

    private void undoColorChange(CropBean cropBean) {
        if (properties.get(cropBean).containsKey(CropBean.PROP_COLOR)) {
            cropBean.setColor((Color) properties.get(cropBean).get(CropBean.PROP_COLOR));
        }
    }

    private class OriginalProperties extends HashMap<String, Object> implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            String propertyName = evt.getPropertyName();
            if (containsKey(propertyName)) {
                removeIfEquals(propertyName, evt.getNewValue());
            } else {
                put(propertyName, evt.getOldValue());
            }
        }

        private void removeIfEquals(String propertyName, Object newValue) {
            if (Objects.equals(get(propertyName), newValue)) {
                remove(propertyName);
            }
        }
    }
}
